/*
 * py_malloc.d - Python libc malloc analysis.
 *               Written for the Python DTrace provider.
 *
 * $Id: py_malloc.d 19 2007-09-12 07:47:59Z brendan $
 *
 * This is an expiremental script to identify who is calling malloc() for
 * memory allocation, and to print distribution plots of the requested bytes.
 * If a malloc() occured while in a Python function, then that function is
 * identified as responsible; else the caller of malloc() is identified as
 * responsible - which will be a function from the Python engine.
 *
 * Filename and function names are printed if available.
 *
 * COPYRIGHT: Copyright (c) 2007 Brendan Gregg.
 *
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 *
 * 09-Sep-2007	Brendan Gregg	Created this.
 * 16-Dec-2008	Tom Erickson	(adapted for Chime)
 */

/*
 * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
 */

python$target:::function-entry
{
	self->file = basename(copyinstr(arg0));
	self->name = copyinstr(arg1);
}

python$target:::function-return
{
	self->file = 0;
	self->name = 0;
}

pid$target:libc:malloc:entry
/self->file != NULL/
{
	@malloc_size[self->file, self->name] = sum(arg0);
	@malloc_dist[self->file, self->name] = quantize(arg0);
}

pid$target:libc:malloc:entry
/self->name == NULL/
{
	@malloc_size[" ", "(Python engine)"] = sum(arg0);
	@malloc_dist[" ", "(Python engine)"] = quantize(arg0);
}
