/*
 * rb_calls.d - count Ruby calls using DTrace.
 *              Written for the Ruby DTrace provider.
 *
 * $Id: rb_calls.d 28 2007-09-13 10:49:37Z brendan $
 *
 * This traces activity from all Ruby programs on the system that are
 * running with Ruby provider support.
 *
 * FIELDS:
 *		FILE		Filename of the Ruby program
 *		TYPE		Type of call (func/obj-new/gc)
 *		NAME		Descriptive name of call
 *		COUNT		Number of calls during sample
 *
 * Filename and method names are printed if available.
 *
 * COPYRIGHT: Copyright (c) 2007 Brendan Gregg.
 *
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 *
 * 09-Sep-2007	Brendan Gregg	Created this.
 * 05-Dec-2008	Tom Erickson	(adapted for Chime)
 */

/*
 * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
 */

ruby*:::function-entry
{
	this->name = strjoin(strjoin(copyinstr(arg0), "::"), copyinstr(arg1));
        @calls[basename(copyinstr(arg2)), "func", this->name] = count();
}

ruby*:::object-create-start
{
	this->name = copyinstr(arg0);
	this->filename = basename(copyinstr(arg1));
	this->filename = this->filename != NULL ? this->filename : ".";
        @calls[this->filename, "obj-new", this->name] = count();
}

ruby*:::object-free
{
	this->name = copyinstr(arg0);
        @calls[".", "obj-free", this->name] = count();
}

ruby*:::gc-begin
{
        @calls[".", "gc", "begin"] = count();
}

ruby*:::raise
{
	this->name = copyinstr(arg0);
        @calls[basename(copyinstr(arg1)), "raise", this->name] = count();
}

ruby*:::rescue
{
        @calls[basename(copyinstr(arg0)), "rescue", "-"] = count();
}
