/*
 * rwtop - display top read/write bytes by process.
 *         Written using DTrace (Solaris 10 3/05).
 *
 * This is measuring reads and writes at the application level. This matches
 * read and write system calls.
 *
 * 24-Jul-2005, ver 0.70
 *
 * SEE ALSO:	iotop
 *
 * INSPIRATION:  top(1) by William LeFebvre
 *
 * COPYRIGHT: Copyright (c) 2005 Brendan Gregg.
 *
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 *
 * Author: Brendan Gregg  [Sydney, Australia]
 *
 * 24-Jul-2005   Brendan Gregg   Created this.
 * 12-Sep-2008   Tom Erickson    Adapted to Chime.
 */

/*
 * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
 */

/*
 * arguments
 */
inline string NAME = $$1;
inline int PID = $2;
inline int OPT_name = (NAME != "");
inline int OPT_pid = (PID > 0);
inline int OPT_filter = (OPT_name || OPT_pid);

/*
 * Check event is being traced
 */
sysinfo:::readch,
sysinfo:::writech
??P0??
{
	/* default is to trace unless filtering, */
	this->ok = OPT_filter ? 0 : 1;

	/* check each filter, */
	(OPT_name == 1 && NAME == execname)? this->ok = 1 : 1;
	(OPT_pid == 1 && PID == pid) ? this->ok = 1 : 1;
}

/*
 * Process event
 */
sysinfo:::readch
/this->ok/
{
	@bytes_r[??T??] = sum(arg0);
	@counts_r[??T??] = count();
}

sysinfo:::writech
/this->ok/
{
	@bytes_w[??T??] = sum(arg0);
	@counts_w[??T??] = count();
}

sysinfo:::readch,
sysinfo:::writech
/this->ok/
{
	@bytes[??T??] = sum(arg0);
	this->ok = 0;
}

??T1 pid
??T2 ppid
??T3 execname

??P0 pid != $pid
